from typing import List, Optional, Union, Set


class AST:
    """Base class for all AST nodes."""

    def pretty(self, indent_str: str = "  ", level: int = 0) -> str:
        """Returns a nicely formatted string representation of this AST node."""
        lines = []
        prefix = indent_str * level
        node_name = self.__class__.__name__
        lines.append(f"{prefix}{node_name}")

        for attr, value in vars(self).items():
            lines.append(self._pretty_attr(attr, value, indent_str, level + 1))

        return "\n".join(line for line in lines if line)

    def _pretty_attr(self, name, value, indent_str, level):
        prefix = indent_str * level
        if isinstance(value, AST):
            return f"{prefix}{name}:\n{value.pretty(indent_str, level + 1)}"

        elif isinstance(value, list):
            lines = [f"{prefix}{name}:"]
            for item in value:
                if isinstance(item, AST):
                    lines.append(item.pretty(indent_str, level + 1))
                else:
                    lines.append(f"{indent_str * (level + 1)}{repr(item)}")
            return "\n".join(lines)

        else:
            return f"{prefix}{name}: {repr(value)}"


class L_cfi(AST):
    __match_args__ = ("declar", "stmt")

    def __init__(self, declar: List["Declar"], stmt: List["Stmt"]) -> None:
        self.declar = declar
        self.stmt = stmt

    def __str__(self):
        return ('\n'.join([str(decl) for decl in self.declar])
                + '\n'
                + '\n'.join([str(stmt) for stmt in self.stmt]))

class Var(AST):
    __match_args__ = ("name",)

    def __init__(self, name: str) -> None:
        self.name = name


class Type(AST):
    pass


class Bool(Type):
    def __str__(self):
        return "bool"


class Int(Type):
    def __str__(self):
        return "int"


class Float(Type):
    def __str__(self):
        return "float"



class Exp(AST):
    pass

class Var(Exp):
    __match_args__ = ("name",)
    def __init__(self, name: str) -> None:
        self.name = name

    def __str__(self):
        return self.name

class Declar(AST):
    __match_args__ = ("lcfi_type", "var")

    def __init__(self, lcfi_type: Type, var: Union[Var, Set[Var]]) -> None:
        self.lcfi_type = lcfi_type
        self.var = var

    def __str__(self):
        if isinstance(self.var, Var):
            return f"{self.lcfi_type} {self.var}"
        else:
            return f"{self.lcfi_type} {', '.join(str(var) for var in self.var)}"

class Constant(Exp):
    __match_args__ = ("value",)

    def __init__(self, value: int | float) -> None:
        self.value = value

    def __str__(self):
        return str(self.value)


class Bools(Exp):
    __match_args__ = ("value",)

    def __init__(self, value: bool) -> None:
        self.value = value

    def __str__(self):
        return "true" if self.value else "false"


class Pred(AST):
    __match_args__ = ("value",)

    def __init__(self, value: Union[Bools, Var]) -> None:
        self.value = value

    def __str__(self):
        return str(self.value)


class Inst(AST):
    pass


class Stmt(AST):
    __match_args__ = ("label", "pred", "inst")

    def __init__(self, label: Optional[str], pred: Pred, inst: Inst) -> None:
        self.label = label
        self.pred = pred
        self.inst = inst

    def __str__(self):
        return f"{self.pred}: {self.inst}"


class PrintInst(Inst):
    __match_args__ = ("string", "exp")

    def __init__(self, string: str, exp: Optional[Exp]) -> None:
        self.string = string
        self.exp = exp

    def __str__(self):
        if self.exp:
            return f"print(\"{self.string}\", {self.exp})"
        else:
            return f"print(\"{self.string}\")"


class ExpInst(Inst):
    __match_args__ = ("exp",)

    def __init__(self, exp: Exp) -> None:
        self.exp = exp

    def __str__(self):
        return str(self.exp)


class Assign(AST):
    __match_args__ = ("var", "exp")

    def __init__(self, var: Var, exp: Exp) -> None:
        self.var = var
        self.exp = exp

    def __str__(self):
        return f"{self.var} = {self.exp}"


class AssignInst(Inst):
    __match_args__ = ("assign",)

    def __init__(self, assign: Assign) -> None:
        self.assign = assign

    def __str__(self):
        return str(self.assign)


class Operator(AST):
    pass


class BoolOperator(Operator):
    pass


class BinaryBoolOperator(BoolOperator):
    pass


class And(BinaryBoolOperator):
    def __str__(self):
        return "&&"


class Or(BinaryBoolOperator):
    def __str__(self):
        return "||"


class UnaryBoolOperator(BoolOperator):
    pass


class Not(UnaryBoolOperator):
    def __str__(self):
        return "!"


class NumOperator(Operator):
    pass


class BinaryNumOperator(NumOperator):
    pass


class Add(BinaryNumOperator):
    def __str__(self):
        return "+"


class Sub(BinaryNumOperator):
    def __str__(self):
        return "-"


class Mult(BinaryNumOperator):
    def __str__(self):
        return "*"


class Div(BinaryNumOperator):
    def __str__(self):
        return "/"

class FloorDiv(BinaryNumOperator):
    def __str__(self):
        return "//"

class Mod(BinaryNumOperator):
    def __str__(self):
        return "%"


class UnaryNumOperator(NumOperator):
    pass


class UAdd(UnaryNumOperator):
    def __str__(self):
        return ""


class USub(UnaryNumOperator):
    def __str__(self):
        return "-"


class Comparator(AST):
    pass


class Eq(Comparator):
    def __str__(self):
        return "=="


class Neq(Comparator):
    def __str__(self):
        return "!="


class Gt(Comparator):
    def __str__(self):
        return ">"


class Ge(Comparator):
    def __str__(self):
        return ">="


class Lt(Comparator):
    def __str__(self):
        return "<"


class Le(Comparator):
    def __str__(self):
        return "<="


class UnaryOp(Exp):
    __match_args__ = ("op", "operand")

    def __init__(self, op: Union[str, Operator], operand: Exp) -> None:
        self.op = op
        self.operand = operand

    def __str__(self):
        leaf_type  = (Constant, Bools, Var)
        operand = self.operand if isinstance(self.operand, leaf_type) else f"({self.operand})"
        return f"{self.op}{operand}"

class BinOp(Exp):
    __match_args__ = ("left", "op", "right")

    def __init__(
        self, left: Exp, op: Union[str, Operator, Comparator], right: Exp
    ) -> None:
        self.left = left
        self.op = op
        self.right = right

    def __str__(self):
        leaf_type = (Constant, Bools, Var)
        left = self.left if isinstance(self.left, leaf_type) else f"({self.left})"
        right = self.right if isinstance(self.right, leaf_type) else f"({self.right})"
        return f"{left} {self.op} {right}"


from typing import Any
from lark import Transformer, v_args
from aporia.aporia_ast import (
    Add,
    And,
    Assign,
    AssignInst,
    BinOp,
    Bool,
    Bools,
    Declar,
    Div,
    FloorDiv,
    Eq,
    ExpInst,
    Float,
    Ge,
    Gt,
    Int,
    L_cfi,
    Constant,
    Le,
    Lt,
    Mod,
    Mult,
    Neq,
    Not,
    Or,
    Pred,
    PrintInst,
    Stmt,
    Sub,
    UAdd,
    USub,
    UnaryOp,
    Var,
)


class LcfiTransformer(Transformer[Any, L_cfi]):
    def start(self, children):
        return children[0]

    def l_cfi(self, children):
        declar_list = []
        stmt_list = []
        for c in children:
            if isinstance(c, Declar):
                declar_list.append(c)
            else:
                stmt_list.append(c)
        return L_cfi(declar_list, stmt_list)

    def declar(self, children):
        t = children[0]
        v = children[1].children
        return Declar(t, v)

    def type(self, children):
        match children[0]:
            case "bool":
                return Bool()
            case "int":
                return Int()
            case "float":
                return Float()

    def stmt(self, children):
        pred = children[0]
        inst = children[1]
        return Stmt(None, pred, inst)

    def LABEL(self, token):
        return str(token)

    def pred(self, children):
        return Pred(children[0])

    def inst(self, children):
        if len(children) == 1:
            if isinstance(children[0], Assign):
                return AssignInst(children[0])
            elif isinstance(children[0], PrintInst):
                return children[0]
            else:
                return ExpInst(children[0])
        else:
            raise ValueError("Unknown inst form: children = {}".format(children))

    def print_inst(self, children):
        string_value = None
        exp_node = None
        if len(children) == 1:
            if isinstance(children[0], str):
                string_value = children[0]
            else:
                exp_node = children[0]
        elif len(children) == 2:
            string_value = children[0]
            exp_node = children[1]
        return PrintInst(string_value, exp_node)

    @v_args(inline=True)
    def STRING(self, s):
        return s[1:-1]

    def assign(self, children):
        v = children[0]
        e = children[1]
        return Assign(v, e)

    def bools(self, children):
        val = children[0]
        return Bools(val == "true")

    @v_args(inline=True)
    def NUMBER(self, token):
        return Constant(int(token))

    @v_args(inline=True)
    def FLOATINGNUMBER(self, token):
        return Constant(float(token))

    def var(self, children):
        return Var(children[0])

    @v_args(inline=True)
    def VAR(self, token):
        return str(token)

    def binop(self, children):
        left, token_op, right = children
        op = self.convert_op(token_op)
        return BinOp(left, op, right)

    def unaryop(self, children):
        (token_op, expr) = children
        op = self.convert_unaryop(token_op)
        return UnaryOp(op, expr)

    def cmpop(self, children):
        left, token_op, right = children
        return BinOp(left, token_op, right)

    @v_args(inline=True)
    def cmp(self, token):
        match token:
            case "<":
                return Lt()
            case "<=":
                return Le()
            case ">":
                return Gt()
            case ">=":
                return Ge()
            case "==":
                return Eq()
            case "!=":
                return Neq()
        return str(token)

    def convert_op(self, token):
        match token:
            case "+":
                return Add()
            case "-":
                return Sub()
            case "*":
                return Mult()
            case "//":
                return FloorDiv()
            case "/":
                return Div()
            case "%":
                return Mod()
            case "||":
                return Or()
            case "&&":
                return And()
            case _:
                raise ValueError(f"Unknown operator: {token}")

    def convert_unaryop(self, token):
        match token:
            case "!":
                return Not()
            case "+":
                return UAdd()
            case "-":
                return USub()
            case _:
                raise ValueError(f"Unknown unary operator: {token}")

    def atom(self, children):
        return children[0]



from functools import singledispatch
from unittest import case

from aporia.aporia_ast import *

# apply functions type check and execute operands

# unary
def apply_not(value):
  if isinstance(value, bool):
    return not value
  else:
    raise Exception("Apply Not: Expected boolean type got value:", value)

def apply_negation(value):
  if isinstance(value, (int, float)):
    return - value
  else:
    raise Exception("Apply Negation: Expected int or float type got value:", value)

# Bool BinOp
def apply_and(left, right):
  if isinstance(left, bool) and isinstance(right, bool):
    return left and right
  else:
    raise Exception("Apply And: Expected bool type got values:", left, right)

def apply_or(left, right):
  if isinstance(left, bool) and isinstance(right, bool):
    return left or right
  else:
    raise Exception("Apply Or: Expected bool type got values:", left, right)

# Number BinOp
def apply_add(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left + right
  else:
    raise Exception("Apply Add: Expected int or float type got values:", left, right)

def apply_sub(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left - right
  else:
    raise Exception("Apply Sub: Expected int or float type got values:", left, right)

def apply_mult(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left * right
  else:
    raise Exception("Apply Mult: Expected int or float type got values:", left, right)

def apply_div(left, right):
  ops = (left, right)
  if not all(isinstance(op, (float, int)) for op in ops):
    raise Exception("Apply Div: Expected int or float type got values:", left, right)
  if right == 0:
    raise Exception("Divide by 0 Error.")
  return left / right if any(isinstance(op, float) for op in ops) else left // right

def apply_floor_div(left, right):
  ops = (left, right)
  if not all(isinstance(op, (float, int)) for op in ops):
    raise Exception("Apply Div: Expected int or float type got values:", left, right)
  if right == 0:
    raise Exception("Divide by 0 Error.")
  return left // right

def apply_mod(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    if right == 0:
      raise Exception("Divide by 0 Error.")
    return left % right
  else:
    raise Exception("Apply Mod: Expected int or float type got values:", left, right)

# comparisons
def compare_eq(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left == right
  else:
    raise Exception("Compare Eq: Expected int or float type got values:", left, right)

def compare_neg(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left != right
  else:
    raise Exception("Compare Eq: Expected int or float type got values:", left, right)

def compare_ge(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left >= right
  else:
    raise Exception("Compare Ge: Expected int or float type got values:", left, right)

def compare_gt(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left > right
  else:
    raise Exception("Compare Gt: Expected int or float type got values:", left, right)

def compare_le(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left <= right
  else:
    raise Exception("Compare Le: Expected int or float type got values:", left, right)

def compare_lt(left, right):
  if isinstance(left, (int, float)) and isinstance(right, (int, float)):
    return left < right
  else:
    raise Exception("Compare Lt: Expected int or float type got values:", left, right)

#print formating
@singledispatch
def format_for_print(value):
  raise NotImplementedError("Unsupported types")

@format_for_print.register
def _(value: bool):
  return str(value).lower()

@format_for_print.register
def _(value: int):
  return str(value)

@format_for_print.register
def _(value: float):
  return str(value)

#assignments
def format_for_assign(to_type, value):
  # match on variable type
  match to_type:
    case Bool():
      match value:
        case bool():
          return value
        case int() | float():
          return value % 2 != 0

    case Float():
      return value
    case Int():
      return int(value)

  raise Exception(f"Variable assignment not allowed: Trying to assign value {value} to type {to_type}.")



import unittest
from tests.util import interpret

class TestMisc(unittest.TestCase):
    def test_empty_program(self):
        source = ""
        self.assertEqual("", interpret(source))

    def test_default_values(self):
        source = """
        int a
        float b
        bool c
        true: print("", a)
        true: print("", b)
        true: print("", c)
        """
        self.assertEqual("0\n0.0\nfalse", interpret(source))


if __name__ == '__main__':
    unittest.main()



from lark import Lark
from aporia.parser.transformer import LcfiTransformer
from aporia.aporia_ast import L_cfi
from pathlib import Path


def parse(lcfi_program: str) -> L_cfi:
    grammar_path = Path(__file__).resolve().parent / "grammar.lark"
    parser = Lark.open(grammar_path, start="start", parser="lalr")
    tree = parser.parse(lcfi_program)
    ast = LcfiTransformer().transform(tree)
    return ast



import unittest
from tests.util import interpret

class TestConditionalExecution(unittest.TestCase):
    def test_useless_statement(self):
        source = """
            bool a
            false: print("this should not be executed")
            true: print("", 0)
        """
        self.assertEqual("0", interpret(source))

    def test_if_else(self):
        source = """
            bool if_condition, else_condition
            int a, b
            true: a = 1
            true: b = 2
            true: if_condition = b < a || !true
            true: else_condition = !if_condition
            if_condition: print("", 0)
            else_condition: print("", 1)
        """
        self.assertEqual("1", interpret(source))


if __name__ == '__main__':
    unittest.main()



import unittest
from tests.util import interpret

class TestArithmetic(unittest.TestCase):
    def test_order_of_operations(self):
        source = """
        true: print("", 1 - (3 + 2) * 2)
        """
        self.assertEqual('-9', interpret(source))

    def test_float(self):
        source = """
        true: print("", 0.1 + 0.8)
        """
        self.assertEqual('0.9', interpret(source))

    def test_multiplication(self):
        source = """
        float a
        int b
        true: a = 1.8
        true: b = 2
        true: print("", (a + 1) * b)
        """
        self.assertEqual('5.6', interpret(source))

    def test_modulo(self):
        source = """
        int a, b
        true: a = 18
        true: b = (a % 12) % 2
        true: print("", b)
        """
        self.assertEqual('0', interpret(source))

    def test_division(self):
        source = """
        float a
        true: a = 1
        true: print("", a / 2.0)
        """
        self.assertEqual('0.5', interpret(source))

    def test_division2(self):
        source = """
        float a
        true: a = 1
        true: print("", a / 2)
        """
        self.assertEqual('0', interpret(source))

    def test_floor_division(self):
        source = """
        int a, b
        true: a = 28
        true: b = a // 5
        true: print("", b)
        """
        self.assertEqual('5', interpret(source))

    def test_floor_division2(self):
        source = """
        int a
        true: a = 1
        true: print("", a / 2)
        """
        self.assertEqual('0', interpret(source))



if __name__ == '__main__':
    unittest.main()



from aporia.aporia_ast import *
import aporia.interpreter.utils as utils

class InterpLcfi:

  def interp(self, p):
    match p:
      case L_cfi():
        env = {}
        output = []
        for d in p.declar:
          self.interp_declar(d, env)
        for s in p.stmt:
          result = self.interp_stmt(s, env)
          if result:
            output.append(result)
        return "\n".join(output)
      case _:
        raise Exception("Function interp received unexpected object.")

  def interp_declar(self, d, env):
    match d:
      case Declar(Bool(), var_list):
        self.init_variables(var_list, {'type': Bool(), 'value': False}, env)
      case Declar(Int(), var_list):
        self.init_variables(var_list, {'type': Int(), 'value': 0}, env)
      case Declar(Float(), var_list):
        self.init_variables(var_list, {'type': Float(), 'value': 0.0}, env)
      case _:
        raise Exception("Function interp_declar received unexpected object.")

  def init_variables(self, var_list, value, env):
    for v in var_list:
      env[v.name] = value.copy()

  def interp_stmt(self, s, env):
    match s:
      case Stmt(label, pred, inst):
        if self.interp_pred(pred.value, env):
          return self.interp_inst(inst, env)
      case _:
        raise Exception("Function interp_stmt received unexpected object.")

  def interp_pred(self, v, env):
    match v:
      case Bools(value):
        return value
      case Var(name):
        value = env[name]["value"]
        if value is None or not isinstance(env[name]["type"], Bool):
          raise Exception(f"Prediction variable {name} is not a boolean.")
        return value

  def interp_inst(self, inst, env):
    match inst:
      case AssignInst(assign):
        if assign.var.name in env.keys():
          env[assign.var.name]["value"] = utils.format_for_assign(env[assign.var.name]["type"], self.interp_exp(assign.exp, env))
        else:
          raise Exception(f"Variable {assign.var.name} was not declared.")
      case PrintInst(string, exp):
        if exp is None and string is None:
          return ""
        else:
          if string is None:
            string = ""
          if exp is None:
            exp = ""
          else:
            exp = utils.format_for_print(self.interp_exp(exp, env))
          return string + exp
      case ExpInst(exp):
        return str(self.interp_exp(exp, env))
      case _:
        raise Exception("Function interp_inst received unexpected object.")
  
  def interp_exp(self, exp, env):
    match exp:
      case Constant(value) | Bools(value):
        return value
      case Var(name):
        return env[name]["value"]
      case UnaryOp(Not(), operand):
        return utils.apply_not(self.interp_exp(operand, env))
      case UnaryOp(USub(), operand):
        return utils.apply_negation(self.interp_exp(operand, env))
      case BinOp(_, BinaryBoolOperator(), _):
        match exp:
          case BinOp(left, And(), right):
            return utils.apply_and(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Or(), right):
            return utils.apply_or(self.interp_exp(left, env), self.interp_exp(right, env))
      case BinOp(_, BinaryNumOperator(), _):
        match exp:
          case BinOp(left, Add(), right):
            return utils.apply_add(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Sub(), right):
            return utils.apply_sub(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Mult(), right):
            return utils.apply_mult(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Div(), right):
            return utils.apply_div(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, FloorDiv(), right):
            return utils.apply_floor_div(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Mod(), right):
            return utils.apply_mod(self.interp_exp(left, env), self.interp_exp(right, env))
      case BinOp(_, Comparator(), _):
        match exp:
          case BinOp(left, Eq(), right):
            return utils.compare_eq(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Neq(), right):
            return utils.compare_neg(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Ge(), right):
            return utils.compare_ge(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Le(), right):
            return utils.compare_le(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Gt(), right):
            return utils.compare_gt(self.interp_exp(left, env), self.interp_exp(right, env))
          case BinOp(left, Lt(), right):
            return utils.compare_lt(self.interp_exp(left, env), self.interp_exp(right, env))




import unittest
from tests.util import interpret

class TestBooleans(unittest.TestCase):
    def test_equal(self):
        source = """
        int a
        true: a = 1.3
        true: print("", a == 1)
        """
        self.assertEqual("true", interpret(source))

    def test_greater_equal(self):
        source = """
        true: print("", 3 >= 3.01)
        """
        self.assertEqual("false", interpret(source))

    def test_smaller_equal(self):
        source = """
        true: print("", 3 <= 3.01)
        """
        self.assertEqual("true", interpret(source))

    def test_smaller(self):
        source = """
        true: print("", 3.01 < 3.01)
        """
        self.assertEqual("false", interpret(source))

    def test_greater(self):
        source = """
        true: print("", !(3.01 > 3.01 || 1 > 3) && 3 > 1)
        """
        self.assertEqual("true", interpret(source))

    def test_negation(self):
        source = """
            bool a, b
            true: a = !(1 == 1)
            true: b = !(!(a))
            true: print("",b)
        """
        self.assertEqual("false", interpret(source))

    def test_and(self):
        source = """
            bool a, b, c
            true: a = true
            true: b = false
            true: c = (a && !b) && b
            true: print("",c)
        """
        self.assertEqual("false", interpret(source))

    def test_or(self):
        source = """
            bool a, b, c
            true: a = true
            true: b = false
            true: c = (!a || b) || b
            true: print("",c)
        """
        self.assertEqual("false", interpret(source))

    def test_complex(self):
        source = """
            bool a, b, c
            true: a = true
            true: b = false
            true: b = a && (b || (c || !a)) && a
            true: print("",b)
        """
        self.assertEqual("false", interpret(source))

if __name__ == '__main__':
    unittest.main()



import unittest
from tests.util import interpret

class TestPrint(unittest.TestCase):
    def test_one_exp_arg(self):
        source = """
        true: print(2)
        """
        self.assertEqual('2', interpret(source))

    def test_str_with_exp(self):
        source = """
        true: print("this is a nine: ", 9)
        """
        self.assertEqual("this is a nine: 9", interpret(source))

    def test_one_str(self):
        source = """
        true: print("hello world")
        """
        self.assertEqual("hello world", interpret(source))


if __name__ == '__main__':
    unittest.main()



import aporia.parser.parser as parser
import aporia.interpreter as interpreter
import argparse
import sys
from pathlib import Path
from timeit import default_timer as timer

def cli():
    argparser = argparse.ArgumentParser(description="Run Aporia programs")
    argparser.add_argument('file', nargs='?', help="Source program which should be interpreted (has the .spp file extension). Can also be provided via stdin")
    argparser.add_argument('-a', '--ast', action='store_true', help="Prints the abstract syntax tree")
    argparser.add_argument('-t', '--time', action='store_true', help="Prints the elapsed time for parsing and interpreting")

    args = argparser.parse_args()

    if args.file:
        source_file = Path(args.file)
        if not source_file.exists():
            print(f"File was not found: {source_file}")
            exit(1)
        with open(source_file) as f:
            source_code = f.read()
    else:
        # Read from stdin if no file argument is given
        if not sys.stdin.isatty():
            source_code = sys.stdin.read().strip()
        else:
            argparser.print_help()
            exit(1)

    start_time = timer()
    ast = parser.parse(source_code)
    result = interpreter.InterpLcfi().interp(ast)
    end_time = timer()

    if args.ast:
        print(f"{"*" * 8} AST START {"*" * 8}")
        print(ast.pretty())
        print(f"{"*" * 8} AST END {"*" * 8}")

    if args.time:
        print(f"Elapsed time: {end_time-start_time}")

    print(result)

if __name__ == "__main__":
    cli()



if __name__ == "__main__":
    cli()

